# Reseda version 1.0.0
# ResedaModelLexer.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Lexer

# 1) Imported modules
# -------------------

from S2MLCore import *
from ResedaToken import *

# 2) Lexer
# --------

# 2.1) Creator
# ------------

class ResedaModelLexer(S2MLModelLexer):

  def __init__(self, engine):
    S2MLModelLexer.__init__(self, engine)

# 2.2) Management of actions
# --------------------------

  def RegisterActions(self):
    S2MLModelLexer.RegisterActions(self)
    self.RegisterAction("-", ResedaModelLexer.ActionMinus)
    self.RegisterAction("<", ResedaModelLexer.ActionLess)
    self.RegisterAction("/", ResedaModelLexer.ActionSlash)
    self.RegisterAction("\\", ResedaModelLexer.ActionAntiSlash)

  def ActionMinus(self, c):
    c = self.GetChar()
    if c==">":
      self.tokenString = "->"
      self.NewToken(ResedaToken.TOKEN_RIGHT_ARROW)
    else:
      self.PutCharBack(c)
      self.tokenString = "-"
      self.NewToken(ResedaToken.TOKEN_MINUS)

  def ActionLess(self, c):
    c1 = self.GetChar()
    if c1=="-":
      c2 = self.GetChar()
      if c2==">":
        self.tokenString = "<->"
        self.NewToken(ResedaToken.TOKEN_LEFT_RIGHT_ARROW)
      else:
        self.PutCharBack(c2)
        self.PutCharBack(c1)
        self.tokenString = "<"
        self.NewToken(ResedaToken.TOKEN_LT)
    elif c1=="=":
      self.tokenString = "<="
      self.NewToken(ResedaToken.TOKEN_LEQ)
    else:
      self.PutCharBack(c1)
      self.tokenString = "<"
      self.NewToken(ResedaToken.TOKEN_LT)

  def ActionSlash(self, c):
    c = self.GetChar()
    if c=="/":
      self.tokenString = "//"
      self.state = ResedaModelLexer.STATE_SINGLE_LINE_COMMENT
    elif c=="*":
      self.tokenString = "/*"
      self.state = ResedaModelLexer.STATE_MULTILINE_COMMENT_1
    elif c=="\\":
      self.tokenString = "/\\"
      self.NewToken(ResedaToken.TOKEN_WEDGE)
    else:
      self.tokenString = "/"
      self.PutCharBack(c)
      self.NewToken(ResedaToken.TOKEN_SLASH)

  def ActionAntiSlash(self, c):
    c = self.GetChar()
    if c=="/":
      self.tokenString = "\\"
      self.NewToken(ResedaToken.TOKEN_VEE)
    else:
      self.tokenString = "\\"
      self.PutCharBack(c)
      self.NewToken(ResedaToken.TOKEN_UNKNOWN)

# 2.3) Management of keywords
# ---------------------------

  def RegisterTokens(self):
    S2MLModelLexer.RegisterTokens(self)

  def RegisterKeywords(self):
    S2MLModelLexer.RegisterKeywords(self)
    self.RegisterKeyword("node", ResedaToken.KEYWORD_NODE)
    self.RegisterKeyword("edge", ResedaToken.KEYWORD_EDGE)
    self.RegisterKeyword("assertion", ResedaToken.KEYWORD_ASSERTION)
    self.RegisterKeyword("Set", ResedaToken.KEYWORD_SET)
    self.RegisterKeyword("parentBlocks", ResedaToken.KEYWORD_PARENT_BLOCKS)
    self.RegisterKeyword("ancestorBlocks", ResedaToken.KEYWORD_ANCESTOR_BLOCKS)
    self.RegisterKeyword("childBlocks", ResedaToken.KEYWORD_CHILD_BLOCKS)
    self.RegisterKeyword("descendantBlocks", ResedaToken.KEYWORD_DESCENDANT_BLOCKS)
    self.RegisterKeyword("childNodes", ResedaToken.KEYWORD_CHILD_NODES)
    self.RegisterKeyword("descendantNodes", ResedaToken.KEYWORD_DESCENDANT_NODES)
    self.RegisterKeyword("childEdges", ResedaToken.KEYWORD_CHILD_EDGES)
    self.RegisterKeyword("descendantEdges", ResedaToken.KEYWORD_DESCENDANT_EDGES)
    self.RegisterKeyword("sourceNodes", ResedaToken.KEYWORD_SOURCE_NODES)
    self.RegisterKeyword("targetNodes", ResedaToken.KEYWORD_TARGET_NODES)
    self.RegisterKeyword("inEdges", ResedaToken.KEYWORD_IN_EDGES)
    self.RegisterKeyword("outEdges", ResedaToken.KEYWORD_OUT_EDGES)
    self.RegisterKeyword("reachableNodes", ResedaToken.KEYWORD_REACHABLE_NODES)
    self.RegisterKeyword("coreachableNodes", ResedaToken.KEYWORD_COREACHABLE_NODES)

