# Reseda version 1.0.0
# ResedaModelChecker.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Undefined references
# 4) Circular definitions
# 5) Types
# 6) Tool specific checks

# 1) Imported modules
# -------------------

from S2MLCore import *
from ResedaModels import *

# 2) Creator
# ----------

class ResedaModelChecker(S2MLModelChecker):
  def __init__(self, engine):
    S2MLModelChecker.__init__(self, engine)

# 3) Undefined references
# -----------------------

  def CheckReferencesInBlock(self, block):
    for element in block.GetElements():
      if element.GetType()==ResedaType.BLOCK:
        previousCheckedBlock = self._checkedBlock
        self._checkedBlock = element
        self.CheckReferencesInBlock(element)
        self._checkedBlock = previousCheckedBlock
      elif element.GetType()==ResedaType.PORT:
        self._checkedElement = element
        self._CheckReferencesInAttributesOfElement(element)
        self._checkedElement = None
      elif element.GetType()==ResedaType.EDGE:
        self._checkedElement = element
        self._CheckReferencesInAttributesOfElement(element)
        self._checkedElement = None
      elif element.GetType()==ResedaType.ARC:
        self._checkedElement = element
        self._CheckReferencesInAttributesOfElement(element)
        self._checkedElement = None
      elif element.GetType()==ResedaType.PARAMETER:
        self._checkedElement = element
        self._CheckReferencesInStochasticExpression(element.GetDefinition())
        self._checkedElement = None
    return self._errors

# 4) Circular definitions
# -----------------------
# Circular definition are checked by the rewriter

# 5) Types
# --------
# Not implemented yet

# 6) Tool specific checks
# -----------------------

  def ToolSpecificCheckModel(self, model):
    # self.CheckInitialValuesOfVariables(model)
    pass

